#pragma once

#ifdef ASDEBUG_EXPORTS
#define ASDEBUG_API __declspec(dllexport)
#else
#define ASDEBUG_API __declspec(dllimport)
#endif

class asDebugCallback
{
public:
	enum ExecuteResult
	{
        executeFinished = 0,
		executeAborted
	};

	virtual void Close()=0;	// After that call, your asDebugApplication would be invalid
	virtual bool Compile(const char*)=0;
	virtual bool Prepare()=0;
	virtual int Execute()=0;	// ExecuteResult
	virtual void Suspend()=0;
	virtual void Abort()=0;

	virtual int GetStackSize()=0;
	virtual int GetStackLine(int)=0;	// -1 for current line
	virtual int GetExceptionLine()=0;
	virtual const char* GetStackFunction(int)=0;	// -1 for current function
	virtual const char* GetExceptionFunction()=0;
	virtual int GetVarCount(int)=0;
	virtual const char* GetVarDeclaration(int,int)=0;	// var,stack index (-1 for current function)
	virtual void* GetVar(int,int)=0;	// var,stack index (-1 for current function)
	virtual const char* DumpVar(const char*,void*)=0;	// return 0 for the default behaviour
};

class ASDEBUG_API asDebugApplication
{
public:
	virtual bool Close()=0;	// Try to close the debugger

	virtual void Show()=0;	// Show the window
	virtual void Hide()=0;	// Hide the window

	virtual bool Load(LPCTSTR)=0;
	virtual bool Save(LPCTSTR)=0;
	virtual bool Set(const char*)=0;	// Sets a specific code into the debugger

	virtual void Start()=0;
	virtual void Stop()=0;

	virtual void OnLine(int)=0;	// Should be called for every line with the line number as a parameter
	virtual void OnException(int)=0;	// Should be called in a case of an exception with the line number as a parameter

	virtual void Output(LPCTSTR)=0;	// Text to write to the output window. Use NULL to clear everything
};

extern "C" ASDEBUG_API asDebugApplication* asDebugCreateApplication(asDebugCallback*);