#pragma once
#include "../ASDebug.h"
#include <angelscript.h>
#include <assert.h>

class asDebugContext : public asDebugCallback, public asIOutputStream 
{
public:
	asDebugContext() : m_pApplication(0), m_pContext(0)
	{
		m_pEngine=asCreateScriptEngine(ANGELSCRIPT_VERSION);
		assert(m_pEngine);
		m_pEngine->SetCommonMessageStream(this);

	}

	virtual ~asDebugContext()
	{
		assert(m_pApplication==0);	// Did you forget to call "Stop"?
		assert(m_pContext==0);
		m_pEngine->Release();
	}

	void Start()
	{
		assert(m_pApplication==0);	// Did you call "Start" twice?
		m_pApplication=asDebugCreateApplication(this);
		m_pApplication->Show();
	}

	bool Stop()
	{
		if (m_pApplication && !m_pApplication->Close())
			return false;
		return true;
	}

	virtual void Close()
	{
		m_pApplication=0;
	}

	virtual bool Compile(const char* szScript)
	{
		m_pEngine->Discard(0);
		m_pEngine->AddScriptSection(0,"main",szScript,strlen(szScript));
		return (m_pEngine->Build(0)>=0);
	}
	virtual bool Prepare()
	{
		m_pContext=m_pEngine->CreateContext();
		m_pContext->SetLineCallback(asMETHOD(asDebugContext,LineCallback),this,asCALL_THISCALL);
		m_pContext->SetExceptionCallback(asMETHOD(asDebugContext,ExceptionCallback),this,asCALL_THISCALL);
		if (m_pContext->Prepare(m_pEngine->GetFunctionIDByDecl(NULL,"void main()"))<0)
		{
			m_pContext->Release();
			return false;
		}
		return true;
	}
	virtual int Execute()
	{
		switch (m_pContext->Execute())
		{
		case asEXECUTION_FINISHED:
			return executeFinished;
		case asEXECUTION_ABORTED:
			return executeAborted;
		}
		return -1;
	}
	virtual void Suspend()
	{
		m_pContext->Suspend();
	}
	virtual void Abort()
	{
		m_pContext->Abort();
		m_pContext->Release();
		m_pContext=0;
	}

	virtual int GetStackSize()
	{
		return m_pContext->GetCallstackSize();
	}
	virtual int GetStackLine(int nIndex)	// -1 for current line
	{
		return (nIndex>=0 ? m_pContext->GetCallstackLineNumber(nIndex) : m_pContext->GetCurrentLineNumber());
	}
	virtual int GetExceptionLine()
	{
		return m_pContext->GetExceptionLineNumber();
	}
	virtual const char* GetStackFunction(int nIndex)	// -1 for current function
	{
		return m_pEngine->GetFunctionDeclaration(nIndex>=0 ? m_pContext->GetCallstackFunction(nIndex) : m_pContext->GetCurrentFunction());
	}
	virtual const char* GetExceptionFunction()
	{
		return m_pEngine->GetFunctionDeclaration(m_pContext->GetExceptionFunction());
	}
	virtual int GetVarCount(int nIndex)
	{
		return m_pContext->GetVarCount(nIndex);
	}
	virtual const char* GetVarDeclaration(int nVar,int nIndex)	// var,stack index (-1 for current function)
	{
		return m_pContext->GetVarDeclaration(nVar,0,nIndex);
	}
	virtual void* GetVar(int nVar,int nIndex)	// var,stack index (-1 for current function)
	{
		return m_pContext->GetVarPointer(nVar,nIndex);
	}
	virtual const char* DumpVar(const char* szType,void* pVar)
	{
		return 0;
	}

	virtual void Write(const char *text)
	{
		m_pApplication->Output(CA2CT(text));	// Need to be converted from LPCSTR to LPCTSTR. Change the line if you are not using ATL
	}

protected:
	asDebugApplication* m_pApplication;

	asIScriptEngine* m_pEngine;
	asIScriptContext* m_pContext;

private:
	void LineCallback(asIScriptContext* pContext)
	{
		m_pApplication->OnLine(pContext->GetCurrentLineNumber());
	}
	void ExceptionCallback(asIScriptContext* pContext)
	{
		m_pApplication->OnException(pContext->GetExceptionLineNumber());
	}
};